@@ -15,21 +15,41 @@ from altscore.borrower_central.model.relationships import RelationshipSync, Rela
 from altscore.borrower_central.model.stages import StageSync, StageAsync
 from altscore.borrower_central.model.steps import StepSync, StepAsync
 from altscore.borrower_central.model.risk_ratings import RiskRatingSync, RiskRatingAsync
+from altscore.borrower_central.model.repayment_risk_ratings import RepaymentRiskRatingSync, RepaymentRiskRatingAsync
+from altscore.borrower_central.model.flags import FlagSync, FlagAsync
 from altscore.borrower_central.model.policy_alerts import AlertSync, AlertAsync
+from altscore.borrower_central.model.metrics import MetricSync, MetricAsync
 
 from altscore.common.http_errors import raise_for_status_improved, retry_on_401, retry_on_401_async
 from altscore.borrower_central.model.store_packages import PackageSync, PackageAsync
 from altscore.borrower_central.model.executions import ExecutionSync, ExecutionAsync
 from altscore.borrower_central.utils import clean_dict, convert_to_dash_case
 
+from loguru import logger
+
+
+class StepDataInBorrower(BaseModel):
+    step_id: str = Field(alias="stepId")
+    order: int = Field(alias="order")
+    key: str = Field(alias="key")
+    created_at: str = Field(alias="createdAt")
+
+    class Config:
+        populate_by_name = True
+        allow_population_by_field_name = True
+        allow_population_by_alias = True
+
 
 class BorrowerAPIDTO(BaseModel):
     id: str = Field(alias="id")
     persona: str = Field(alias="persona")
     avatar_url: Optional[str] = Field(alias="avatarUrl")
     label: Optional[str] = Field(alias="label")
-    flag: Optional[str] = Field(alias="flag", default=None)
     tags: List[str] = Field(alias="tags", default=[])
+    flag: Optional[str] = Field(alias="flag", default=None)
+    risk_rating: Optional[str] = Field(alias="riskRating", default=None)
+    repayment_risk_rating: Optional[int] = Field(alias="repaymentRiskRating", default=None)
+    current_step: Optional[StepDataInBorrower] = Field(alias="currentStep", default=None)
     cms_client_ids: Optional[List[str]] = Field(alias="cmsClientIds", default=[])
     created_at: str = Field(alias="createdAt")
     updated_at: Optional[str] = Field(alias="updatedAt")
@@ -93,8 +113,9 @@ class BorrowerSummaryAPIDTO(BaseModel):
     points_of_contact: Optional[List[SimplifiedPointOfContact]] = Field(alias="pointsOfContact", default=[])
     tags: List[str] = Field(alias="tags", default=[])
     stage: Optional[str] = Field(alias="stage", default=None)
-    current_step: Optional[CurrentStep] = Field(alias="currentStep", default=None)
     risk_rating: Optional[str] = Field(alias="riskRating", default=None)
+    repayment_risk_rating: Optional[int] = Field(alias="repaymentRiskRating", default=None)
+    current_step: Optional[StepDataInBorrower] = Field(alias="currentStep", default=None)
     created_at: str = Field(alias="createdAt")
     updated_at: Optional[str] = Field(alias="updatedAt")
 
@@ -107,6 +128,8 @@ class BorrowerSummaryAPIDTO(BaseModel):
 class CreateBorrowerDTO(BaseModel):
     persona: str = Field(alias="persona")
     label: Optional[str] = Field(alias="label")
+    risk_rating: Optional[str] = Field(alias="riskRating", default=None)
+    repayment_risk_rating: Optional[int] = Field(alias="repaymentRiskRating", default=None)
     flag: Optional[str] = Field(alias="flag", default=None)
     tags: List[str] = Field(alias="tags", default=[])
 
@@ -118,7 +141,6 @@ class CreateBorrowerDTO(BaseModel):
 
 class UpdateBorrowerDTO(BaseModel):
     label: Optional[str] = Field(alias="label", default=None)
-    flag: Optional[str] = Field(alias="flag", default=None)
     tags: List[str] = Field(alias="tags", default=[])
 
     class Config:
@@ -177,6 +199,21 @@ class BorrowerBase:
         }
         return f"{self.base_url}/v1/identities", clean_dict(query)
 
+    def _metrics(
+            self, borrower_id: str, sort_by: Optional[str] = None,
+            key: Optional[str] = None, per_page: Optional[int] = None, page: Optional[int] = None,
+            sort_direction: Optional[str] = None
+    ) -> (str, dict):
+        query = {
+            "borrower-id": borrower_id,
+            "key": key,
+            "sort-by": sort_by,
+            "per-page": per_page,
+            "page": page,
+            "sort-direction": sort_direction
+        }
+        return f"{self.base_url}/v1/metrics", clean_dict(query)
+
     def _documents(
             self, borrower_id: str, key: Optional[str] = None, sort_by: Optional[str] = None,
             per_page: Optional[int] = None, page: Optional[int] = None, sort_direction: Optional[str] = None
@@ -192,7 +229,7 @@ class BorrowerBase:
         return f"{self.base_url}/v1/documents", clean_dict(query)
 
     def _points_of_contact(
-            self, borrower_id: str, contact_method: Optional[str], priority: Optional[int] = None,
+            self, borrower_id: str, contact_method: Optional[str] = None, priority: Optional[int] = None,
             sort_by: Optional[str] = None, per_page: Optional[int] = None, page: Optional[int] = None,
             sort_direction: Optional[str] = None
     ) -> (str, dict):
@@ -238,7 +275,7 @@ class BorrowerBase:
         return f"{self.base_url}/v1/borrower-fields", clean_dict(query)
 
     def _packages(
-            self, borrower_id: str, source_id: Optional[str], sort_by: Optional[str] = None,
+            self, borrower_id: str, source_id: Optional[str] = None, sort_by: Optional[str] = None,
             per_page: Optional[int] = None, page: Optional[int] = None, sort_direction: Optional[str] = None
     ) -> (str, dict):
         query = {
@@ -252,7 +289,7 @@ class BorrowerBase:
         return f"{self.base_url}/v1/stores/packages", clean_dict(query)
 
     def _executions(
-            self, borrower_id: str, execution_id: Optional[str], workflow_id: Optional[str],
+            self, borrower_id: str, execution_id: Optional[str] = None, workflow_id: Optional[str] = None,
             sort_by: Optional[str] = None, per_page: Optional[int] = None, page: Optional[int] = None,
             sort_direction: Optional[str] = None
     ) -> (str, dict):
@@ -268,9 +305,10 @@ class BorrowerBase:
         return f"{self.base_url}/v1/executions", clean_dict(query)
 
     def _alerts(
-            self, borrower_id: str, alert_id: Optional[str], rule_id: Optional[str],
-            rule_code: Optional[str], level: Optional[str], reference_id: Optional[str],
-            is_dismissed: Optional[bool]
+            self, borrower_id: str, alert_id: Optional[str] = None, rule_id: Optional[str] = None,
+            rule_code: Optional[str] = None, level: Optional[str] = None, reference_id: Optional[str] = None,
+            is_acknowledged: Optional[bool] = None, sort_by: Optional[str] = None, per_page: Optional[int] = None,
+            page: Optional[int] = None, sort_direction: Optional[str] = None
     ) -> (str, dict):
         query = {
             "borrower-id": borrower_id,
@@ -279,7 +317,11 @@ class BorrowerBase:
             "rule-code": rule_code,
             "level": level,
             "reference-id": reference_id,
-            "is-dismissed": is_dismissed
+            "is-acknowledged": is_acknowledged,
+            "sort-by": sort_by,
+            "per-page": per_page,
+            "page": page,
+            "sort-direction": sort_direction
         }
         return f"{self.base_url}/v1/alerts", clean_dict(query)
 
@@ -338,7 +380,6 @@ class BorrowersAsyncModule:
                 headers=self.build_headers(),
                 timeout=120,
             )
-            raise_for_status_improved(response)
             if response.status_code == 200:
                 return BorrowerAsync(
                     base_url=self.altscore_client._borrower_central_base_url,
@@ -346,7 +387,9 @@ class BorrowersAsyncModule:
                     renew_token=self.renew_token,
                     data=BorrowerAPIDTO.parse_obj(response.json())
                 )
-            return None
+            elif response.status_code in [404]:
+                return None
+            raise_for_status_improved(response)
 
     @retry_on_401_async
     async def find_one_by_identity(self, identity_key: str, identity_value: str):
@@ -417,6 +460,74 @@ class BorrowersAsyncModule:
             raise_for_status_improved(response)
             return [BorrowerSummaryAPIDTO.parse_obj(e) for e in response.json()]
 
+    @retry_on_401_async
+    async def retrieve_all(self, **kwargs):
+        query_params = {}
+        per_page = 10
+        for k, v in kwargs.items():
+            if v is not None:
+                query_params[convert_to_dash_case(k)] = v
+        query_params["per-page"] = per_page
+        async with httpx.AsyncClient(base_url=self.altscore_client._borrower_central_base_url) as client:
+            response = await client.get(
+                f"/v1/borrowers",
+                params=query_params,
+                headers=self.build_headers(),
+                timeout=30
+            )
+            raise_for_status_improved(response)
+            total_count = int(response.headers["x-total-count"])
+        resources = []
+        # TODO: this is not optimal, we should use asyncio.gather and a batch size
+        total_pages = (total_count // per_page) + 1
+        if total_pages > 1:
+            pages = range(1, total_pages + 1)
+        else:
+            pages = [1]
+        for page in pages:
+            resources.append(await self.query(page=page, per_page=per_page, **kwargs))
+        resources = [item for sublist in resources for item in sublist]
+        return resources
+
+    @retry_on_401_async
+    async def summary_retrieve_all(self, **kwargs):
+        query_params = {}
+        kwargs_allowed = {}
+        per_page = 50
+        for k, v in kwargs.items():
+            if k in ["by", "search", "borrower_id"]:
+                logger.warning(f"Skipping {k} as it is not allowed in summary retrieve all")
+                continue
+            if v is not None:
+                kwargs_allowed[k] = v
+                query_params[convert_to_dash_case(k)] = v
+        query_params["per-page"] = per_page
+        async with httpx.AsyncClient(base_url=self.altscore_client._borrower_central_base_url) as client:
+            response = await client.get(
+                f"/v1/borrowers-summary",
+                params=query_params,
+                headers=self.build_headers(),
+                timeout=30
+            )
+            raise_for_status_improved(response)
+            total_count = int(response.headers["x-total-count"])
+        total_pages = (total_count // per_page) + 1
+        if total_pages > 1:
+            pages = range(1, total_pages + 1)
+        else:
+            pages = [1]
+
+        results = []
+        page_chunks = [pages[i:i + 5] for i in range(0, len(pages), 5)]
+        for page_chunk in page_chunks:
+            calls = []
+            for page in page_chunk:
+                calls.append(self.query_summary(page=page, per_page=per_page, **kwargs_allowed))
+            results += await asyncio.gather(*calls)
+
+        results = [item for sublist in results for item in sublist]
+        return results
+
 
 class BorrowersSyncModule:
 
@@ -472,7 +583,6 @@ class BorrowersSyncModule:
                 headers=self.build_headers(),
                 timeout=120
             )
-            raise_for_status_improved(response)
             if response.status_code == 200:
                 return BorrowerSync(
                     base_url=self.altscore_client._borrower_central_base_url,
@@ -480,7 +590,9 @@ class BorrowersSyncModule:
                     renew_token=self.renew_token,
                     data=BorrowerAPIDTO.parse_obj(response.json())
                 )
-            return None
+            elif response.status_code in [404]:
+                return None
+            raise_for_status_improved(response)
 
     @retry_on_401
     def find_one_by_identity(self, identity_key: str, identity_value: str):
@@ -551,6 +663,71 @@ class BorrowersSyncModule:
             raise_for_status_improved(response)
             return [BorrowerSummaryAPIDTO.parse_obj(e) for e in response.json()]
 
+    @retry_on_401
+    def retrieve_all(self, **kwargs):
+        query_params = {}
+        per_page = 10
+        for k, v in kwargs.items():
+            if v is not None:
+                query_params[convert_to_dash_case(k)] = v
+        query_params["per-page"] = per_page
+        with httpx.Client(base_url=self.altscore_client._borrower_central_base_url) as client:
+            response = client.get(
+                f"/v1/borrowers",
+                params=query_params,
+                headers=self.build_headers(),
+                timeout=30
+            )
+            raise_for_status_improved(response)
+            total_count = int(response.headers["x-total-count"])
+        resources = []
+        # TODO: this is not optimal, we should use asyncio.gather and a batch size
+        total_pages = (total_count // per_page) + 1
+        if total_pages > 1:
+            pages = range(1, total_pages + 1)
+        else:
+            pages = [1]
+        for page in pages:
+            r = self.query(page=page, per_page=per_page, **kwargs)
+            resources.append(r)
+        resources = [item for sublist in resources for item in sublist]
+        return resources
+
+    @retry_on_401
+    def summary_retrieve_all(self, **kwargs):
+        query_params = {}
+        kwargs_allowed = {}
+        per_page = 50
+        for k, v in kwargs.items():
+            if k in ["by", "search", "borrower_id"]:
+                logger.warning(f"Skipping {k} as it is not allowed in summary retrieve all")
+                continue
+            if v is not None:
+                kwargs_allowed[k] = v
+                query_params[convert_to_dash_case(k)] = v
+        query_params["per-page"] = per_page
+        with httpx.Client(base_url=self.altscore_client._borrower_central_base_url) as client:
+            response = client.get(
+                f"/v1/borrowers-summary",
+                params=query_params,
+                headers=self.build_headers(),
+                timeout=30
+            )
+            raise_for_status_improved(response)
+            total_count = int(response.headers["x-total-count"])
+        resources = []
+        # TODO: this is not optimal, we should use asyncio.gather and a batch size
+        total_pages = (total_count // per_page) + 1
+        if total_pages > 1:
+            pages = range(1, total_pages + 1)
+        else:
+            pages = [1]
+        for page in pages:
+            r = self.query_summary(page=page, per_page=per_page, **kwargs_allowed)
+            resources.append(r)
+        resources = [item for sublist in resources for item in sublist]
+        return resources
+
 
 class BorrowerAsync(BorrowerBase):
     data: BorrowerAPIDTO
@@ -645,6 +822,21 @@ class BorrowerAsync(BorrowerBase):
                 data=response.json()
             )
 
+    @retry_on_401_async
+    async def get_repayment_risk_rating(self) -> RepaymentRiskRatingAsync:
+        async with httpx.AsyncClient(base_url=self.base_url) as client:
+            response = await client.get(
+                f"{self.base_url}/v1/borrowers/{self.data.id}/repayment-risk-rating",
+                headers=self._header_builder()
+            )
+            raise_for_status_improved(response)
+            return RepaymentRiskRatingAsync(
+                base_url=self.base_url,
+                header_builder=self._header_builder,
+                renew_token=self.renew_token,
+                data=response.json()
+            )
+
     @retry_on_401_async
     async def set_risk_rating(self, risk_rating: str, reference_id: Optional[str] = None):
         async with httpx.AsyncClient(base_url=self.base_url) as client:
@@ -659,6 +851,34 @@ class BorrowerAsync(BorrowerBase):
             raise_for_status_improved(response)
             return None
 
+    @retry_on_401_async
+    async def set_repayment_risk_rating(self, risk_rating: str, reference_id: Optional[str] = None):
+        async with httpx.AsyncClient(base_url=self.base_url) as client:
+            response = await client.put(
+                f"{self.base_url}/v1/borrowers/{self.data.id}/repayment-risk-rating",
+                headers=self._header_builder(),
+                json={
+                    "value": risk_rating,
+                    "referenceId": reference_id
+                }
+            )
+            raise_for_status_improved(response)
+            return None
+
+    @retry_on_401_async
+    async def set_flag(self, flag: str, reference_id: Optional[str] = None):
+        async with httpx.AsyncClient(base_url=self.base_url) as client:
+            response = await client.put(
+                f"{self.base_url}/v1/borrowers/{self.data.id}/flag",
+                headers=self._header_builder(),
+                json={
+                    "value": flag,
+                    "referenceId": reference_id
+                }
+            )
+            raise_for_status_improved(response)
+            return None
+
     @retry_on_401_async
     async def get_documents(self, **kwargs) -> List[DocumentAsync]:
         async with httpx.AsyncClient(base_url=self.base_url) as client:
@@ -695,6 +915,25 @@ class BorrowerAsync(BorrowerBase):
                 data=data[0]
             )
 
+    @retry_on_401_async
+    async def get_metric_by_key(self, key: str) -> Optional[MetricAsync]:
+        async with httpx.AsyncClient(base_url=self.base_url) as client:
+            url, query = self._metrics(self.data.id, key=key)
+            response = await client.get(
+                url,
+                headers=self._header_builder(),
+                params=query
+            )
+            data = response.json()
+            if len(data) == 0:
+                return None
+            return MetricAsync(
+                base_url=self.base_url,
+                header_builder=self._header_builder,
+                renew_token=self.renew_token,
+                data=data[0]
+            )
+
     @retry_on_401_async
     async def get_borrower_field_by_key(self, key: str) -> Optional[BorrowerFieldAsync]:
         async with httpx.AsyncClient(base_url=self.base_url) as client:
@@ -782,6 +1021,23 @@ class BorrowerAsync(BorrowerBase):
                 data=borrower_field_data
             ) for borrower_field_data in data]
 
+    @retry_on_401_async
+    async def get_metrics(self, **kwargs) -> List[MetricAsync]:
+        async with httpx.AsyncClient(base_url=self.base_url) as client:
+            url, query = self._metrics(self.data.id, **kwargs)
+            response = await client.get(
+                url,
+                headers=self._header_builder(),
+                params=query
+            )
+            data = response.json()
+            return [MetricAsync(
+                base_url=self.base_url,
+                header_builder=self._header_builder,
+                renew_token=self.renew_token,
+                data=borrower_field_data
+            ) for borrower_field_data in data]
+
     @retry_on_401_async
     async def get_authorizations(self, **kwargs) -> List[AuthorizationAsync]:
         async with httpx.AsyncClient(base_url=self.base_url) as client:
@@ -909,35 +1165,51 @@ class BorrowerAsync(BorrowerBase):
         else:
             return points_of_contact[0]
 
-    @retry_on_401
+    @retry_on_401_async
     async def map_identities_and_fields_onto_dict(self, mapping_dict: dict):
-        identities_to_query = {k: 1 for k in mapping_dict.keys() if k.startswith("identity.")}
+        identities_to_query = {k: 1 for k in mapping_dict.values() if k.startswith("identity.")}
         malformed_identities = [k for k in identities_to_query if len(k.split(".")[-1]) == 0]
-        borrower_fields_to_query = {k: 1 for k in mapping_dict.keys() if k.startswith("borrower_field.")}
+        borrower_fields_to_query = {k: 1 for k in mapping_dict.values() if k.startswith("borrower_field.")}
         malformed_fields = [k for k in borrower_fields_to_query if len(k.split(".")[-1]) == 0]
         if len(malformed_fields + malformed_identities) > 0:
             raise ValueError(f"Found malformed keys: {malformed_fields + malformed_identities}")
         calls = []
         for identity_key in identities_to_query:
             calls.append(
-                self.get_identity_by_key(identity_key)
+                self.get_identity_by_key(identity_key.replace("identity.", ""))
             )
         for field_key in borrower_fields_to_query:
             calls.append(
-                self.get_borrower_field_by_key(field_key)
+                self.get_borrower_field_by_key(field_key.replace("borrower_field.", ""))
             )
-        value_maps = {}
         calls = await asyncio.gather(*calls)
+        value_maps = {}
         for element in calls:
-            if identities_to_query.get(element.data.key) is not None:
+            if element is None:
+                pass
+            elif element.resource == "identities":
                 value_maps[f"identity.{element.data.key}"] = element.data.value
-            elif borrower_fields_to_query.get(element.data.key) is not None:
+            elif element.resource == "borrower-fields":
                 value_maps[f"borrower_field.{element.data.key}"] = element.data.value
         mapped_dict = {}
-        for k, v_map in mapping_dict:
+        for k, v_map in mapping_dict.items():
             mapped_dict[k] = value_maps.get(v_map)
         return mapped_dict
 
+    @retry_on_401_async
+    async def send_sms(self, message: str, point_of_contact_id: Optional[str] = None):
+        async with httpx.AsyncClient(base_url=self.base_url) as client:
+            response = await client.post(
+                f"{self.base_url}/v1/borrowers/{self.data.id}/communications/sms",
+                headers=self._header_builder(),
+                json={
+                    "message": message,
+                    "pointOfContactId": point_of_contact_id
+                }
+            )
+            raise_for_status_improved(response)
+            return None
+
     def __str__(self):
         return str(self.data)
 
@@ -1024,7 +1296,7 @@ class BorrowerSync(BorrowerBase):
             return None
 
     @retry_on_401
-    def get_risk_rating(self):
+    def get_risk_rating(self) -> RiskRatingSync:
         with httpx.Client(base_url=self.base_url) as client:
             response = client.get(
                 f"{self.base_url}/v1/borrowers/{self.data.id}/risk-rating",
@@ -1038,6 +1310,21 @@ class BorrowerSync(BorrowerBase):
                 data=response.json()
             )
 
+    @retry_on_401
+    def get_repayment_risk_rating(self) -> RepaymentRiskRatingSync:
+        with httpx.Client(base_url=self.base_url) as client:
+            response = client.get(
+                f"{self.base_url}/v1/borrowers/{self.data.id}/repayment-risk-rating",
+                headers=self._header_builder()
+            )
+            raise_for_status_improved(response)
+            return RepaymentRiskRatingSync(
+                base_url=self.base_url,
+                header_builder=self._header_builder,
+                renew_token=self.renew_token,
+                data=response.json()
+            )
+
     @retry_on_401
     def set_risk_rating(self, risk_rating: str, reference_id: Optional[str] = None):
         with httpx.Client(base_url=self.base_url) as client:
@@ -1052,6 +1339,34 @@ class BorrowerSync(BorrowerBase):
             raise_for_status_improved(response)
             return None
 
+    @retry_on_401
+    def set_repayment_risk_rating(self, risk_rating: str, reference_id: Optional[str] = None):
+        with httpx.Client(base_url=self.base_url) as client:
+            response = client.put(
+                f"{self.base_url}/v1/borrowers/{self.data.id}/repayment-risk-rating",
+                headers=self._header_builder(),
+                json={
+                    "value": risk_rating,
+                    "referenceId": reference_id
+                }
+            )
+            raise_for_status_improved(response)
+            return None
+
+    @retry_on_401
+    def set_flag(self, flag: str, reference_id: Optional[str] = None):
+        with httpx.Client(base_url=self.base_url) as client:
+            response = client.put(
+                f"{self.base_url}/v1/borrowers/{self.data.id}/flag",
+                headers=self._header_builder(),
+                json={
+                    "value": flag,
+                    "referenceId": reference_id
+                }
+            )
+            raise_for_status_improved(response)
+            return None
+
     @retry_on_401
     def get_documents(self, **kwargs) -> List[DocumentSync]:
         with httpx.Client(base_url=self.base_url) as client:
@@ -1088,6 +1403,25 @@ class BorrowerSync(BorrowerBase):
                 data=data[0]
             )
 
+    @retry_on_401
+    def get_metric_by_key(self, key: str) -> Optional[MetricSync]:
+        with httpx.Client(base_url=self.base_url) as client:
+            url, query = self._metrics(self.data.id, key=key)
+            response = client.get(
+                url,
+                headers=self._header_builder(),
+                params=query
+            )
+            data = response.json()
+            if len(data) == 0:
+                return None
+            return MetricSync(
+                base_url=self.base_url,
+                header_builder=self._header_builder,
+                renew_token=self.renew_token,
+                data=data[0]
+            )
+
     @retry_on_401
     def get_borrower_field_by_key(self, key: str) -> Optional[BorrowerFieldSync]:
         with httpx.Client(base_url=self.base_url) as client:
@@ -1175,6 +1509,23 @@ class BorrowerSync(BorrowerBase):
                 data=borrower_field_data
             ) for borrower_field_data in data]
 
+    @retry_on_401
+    def get_metrics(self, **kwargs) -> List[MetricSync]:
+        with httpx.Client(base_url=self.base_url) as client:
+            url, query = self._metrics(self.data.id, **kwargs)
+            response = client.get(
+                url,
+                headers=self._header_builder(),
+                params=query
+            )
+            data = response.json()
+            return [MetricSync(
+                base_url=self.base_url,
+                header_builder=self._header_builder,
+                renew_token=self.renew_token,
+                data=borrower_field_data
+            ) for borrower_field_data in data]
+
     @retry_on_401
     def get_authorizations(self, **kwargs) -> List[AuthorizationSync]:
         with httpx.Client(base_url=self.base_url) as client:
@@ -1304,32 +1655,48 @@ class BorrowerSync(BorrowerBase):
 
     @retry_on_401
     def map_identities_and_fields_onto_dict(self, mapping_dict: dict):
-        identities_to_query = {k: 1 for k in mapping_dict.keys() if k.startswith("identity.")}
+        identities_to_query = {k: 1 for k in mapping_dict.values() if k.startswith("identity.")}
         malformed_identities = [k for k in identities_to_query if len(k.split(".")[-1]) == 0]
-        borrower_fields_to_query = {k: 1 for k in mapping_dict.keys() if k.startswith("borrower_field.")}
+        borrower_fields_to_query = {k: 1 for k in mapping_dict.values() if k.startswith("borrower_field.")}
         malformed_fields = [k for k in borrower_fields_to_query if len(k.split(".")[-1]) == 0]
         if len(malformed_fields + malformed_identities) > 0:
             raise ValueError(f"Found malformed keys: {malformed_fields + malformed_identities}")
         calls = []
         for identity_key in identities_to_query:
             calls.append(
-                self.get_identity_by_key(identity_key)
+                self.get_identity_by_key(identity_key.replace("identity.", ""))
             )
         for field_key in borrower_fields_to_query:
             calls.append(
-                self.get_borrower_field_by_key(field_key)
+                self.get_borrower_field_by_key(field_key.replace("borrower_field.", ""))
             )
         value_maps = {}
         for element in calls:
-            if identities_to_query.get(element.data.key) is not None:
+            if element is None:
+                pass
+            elif element.resource == "identities":
                 value_maps[f"identity.{element.data.key}"] = element.data.value
-            elif borrower_fields_to_query.get(element.data.key) is not None:
+            elif element.resource == "borrower-fields":
                 value_maps[f"borrower_field.{element.data.key}"] = element.data.value
         mapped_dict = {}
-        for k, v_map in mapping_dict:
+        for k, v_map in mapping_dict.items():
             mapped_dict[k] = value_maps.get(v_map)
         return mapped_dict
 
+    @retry_on_401
+    def send_sms(self, message: str, point_of_contact_id: Optional[str] = None):
+        with httpx.Client(base_url=self.base_url) as client:
+            response = client.post(
+                f"{self.base_url}/v1/borrowers/{self.data.id}/communications/sms",
+                headers=self._header_builder(),
+                json={
+                    "message": message,
+                    "pointOfContactId": point_of_contact_id
+                }
+            )
+            raise_for_status_improved(response)
+            return None
+
     def __str__(self):
         return str(self.data)
 
