@@ -13,8 +13,10 @@ class CreateExecutionDTO(BaseModel):
     workflow_id: Optional[str] = Field(alias="workflowId", default=None)
     workflow_alias: Optional[str] = Field(alias="workflowAlias", default=None)
     workflow_version: Optional[str] = Field(alias="workflowVersion", default=None)
+    workflow_type: Optional[str] = Field(alias="workflowType", default=None)
     borrower_id: Optional[str] = Field(alias="borrowerId", default=None)
     billable_id: Optional[str] = Field(alias="billableId", default=None)
+    batch_id: Optional[str] = Field(alias="batchId", default=None)
 
     class Config:
         populate_by_name = True
@@ -28,6 +30,7 @@ class ExecutionAPIDTO(BaseModel):
     workflow_id: str = Field(alias="workflowId")
     workflow_alias: str = Field(alias="workflowAlias")
     workflow_version: str = Field(alias="workflowVersion")
+    workflow_type: Optional[str] = Field(alias="workflowType")
     batch_id: Optional[str] = Field(alias="batchId")
     billable_id: Optional[str] = Field(alias="billableId")
     borrower_id: Optional[str] = Field(alias="borrowerId")
@@ -50,6 +53,7 @@ class ExecutionOutputDataAPIDTO(BaseModel):
     workflow_id: str = Field(alias="workflowId")
     workflow_alias: str = Field(alias="workflowAlias")
     workflow_version: str = Field(alias="workflowVersion")
+    workflow_type: Optional[str] = Field(alias="workflowType")
     status: Optional[str] = Field(alias="status", default=None)
     is_success: Optional[bool] = Field(alias="isSuccess")
     output: Any = Field(alias="output")
