@@ -6,17 +6,28 @@ from altscore.borrower_central.model.generics import GenericSyncResource, Generi
     GenericSyncModule, GenericAsyncModule
 
 
+class WorkflowSchedule(BaseModel):
+    cron: str = Field(alias="cron")
+    utc_delta_hours: int = Field(alias="utcDeltaHours", default=0, ge=-12, le=14)
+
+    class Config:
+        populate_by_name = True
+        allow_population_by_field_name = True
+        allow_population_by_alias = True
+
+
 class WorkflowDataAPIDTO(BaseModel):
     id: str = Field(alias="id")
-    execution_mode: str = Field(alias="executionMode")
+    execution_mode: Optional[str] = Field(alias="executionMode", default=None)
     alias: str = Field(alias="alias")
     version: str = Field(alias="version")
     label: Optional[str] = Field(alias="label")
     type: Optional[str] = Field(alias="type")
     description: Optional[str] = Field(alias="description")
     context: Optional[str] = Field(alias="context")
-    input_schema: Optional[dict] = Field(alias="inputSchema")
+    input_schema: Optional[str] = Field(alias="inputSchema", default=None)
     flow_definition: Optional[dict] = Field(alias="flowDefinition")
+    schedule: Optional[WorkflowSchedule] = Field(alias="schedule", default=None)
     created_at: str = Field(alias="createdAt")
     updated_at: Optional[str] = Field(alias="updatedAt")
 
@@ -35,11 +46,14 @@ class CreateWorkflowDTO(BaseModel):
     label: Optional[str] = Field(alias="label")
     alias: str = Field(alias="alias")
     version: str = Field(alias="version")
-    execution_mode: str = Field(alias="executionMode")
+    type: Optional[str] = Field(alias="type", default=None)
+    execution_mode: Optional[str] = Field(alias="executionMode", default=None)
     description: Optional[str] = Field(alias="description")
     context: Optional[str] = Field(alias="context", default=None)
-    flow_definition: Optional[dict] = Field(alias="flowDefinition")
+    flow_definition: Optional[dict] = Field(alias="flowDefinition", default=None)
+    input_schema: Optional[str] = Field(alias="inputSchema", default=None)
     route: Optional[Lambda] = Field(alias="route", default=None)
+    schedule: Optional[WorkflowSchedule] = Field(alias="schedule", default=None)
 
     class Config:
         populate_by_name = True
@@ -50,8 +64,11 @@ class CreateWorkflowDTO(BaseModel):
 class UpdateWorkflowDTO(BaseModel):
     label: Optional[str] = Field(alias="label")
     description: Optional[str] = Field(alias="description")
+    type: Optional[str] = Field(alias="type", default=None)
     route: Optional[Lambda] = Field(alias="route", default=None)
-    flow_definition: Optional[dict] = Field(alias="flowDefinition")
+    flow_definition: Optional[dict] = Field(alias="flowDefinition", default=None)
+    input_schema: Optional[str] = Field(alias="inputSchema", default=None)
+    schedule: Optional[WorkflowSchedule] = Field(alias="schedule", default=None)
 
     class Config:
         populate_by_name = True
@@ -129,13 +146,21 @@ class WorkflowsSyncModule(GenericSyncModule):
                 workflow_id: Optional[str] = None,
                 workflow_alias: Optional[str] = None,
                 workflow_version: Optional[str] = None,
+                execution_mode: Optional[str] = None,
+                batch_id: Optional[str] = None
                 ):
+        headers = self.build_headers()
+        if execution_mode is not None:
+            headers["X-Execution-Mode"] = execution_mode
+        if batch_id is not None:
+            headers["X-Batch-Id"] = batch_id
+
         if workflow_id is not None:
             with httpx.Client(base_url=self.altscore_client._borrower_central_base_url) as client:
                 response = client.post(
                     f"/v1/workflows/{workflow_id}/execute",
                     json=workflow_input,
-                    headers=self.build_headers(),
+                    headers=headers,
                     timeout=900
                 )
                 raise_for_status_improved(response)
@@ -146,7 +171,7 @@ class WorkflowsSyncModule(GenericSyncModule):
                 response = client.post(
                     f"/v1/workflows/{workflow_alias}/{workflow_version}/execute",
                     json=workflow_input,
-                    headers=self.build_headers(),
+                    headers=headers,
                     timeout=900
                 )
                 raise_for_status_improved(response)
@@ -197,13 +222,17 @@ class WorkflowsAsyncModule(GenericAsyncModule):
                       workflow_id: Optional[str] = None,
                       workflow_alias: Optional[str] = None,
                       workflow_version: Optional[str] = None,
+                      execution_mode: Optional[str] = None
                       ):
+        headers = self.build_headers()
+        if execution_mode is not None:
+            headers["X-Execution-Mode"] = execution_mode
         if workflow_id is not None:
             async with httpx.AsyncClient(base_url=self.altscore_client._borrower_central_base_url) as client:
                 response = await client.post(
                     f"/v1/workflows/{workflow_id}/execute",
                     json=workflow_input,
-                    headers=self.build_headers(),
+                    headers=headers,
                     timeout=900
                 )
                 raise_for_status_improved(response)
@@ -214,7 +243,7 @@ class WorkflowsAsyncModule(GenericAsyncModule):
                 response = await client.post(
                     f"/v1/workflows/{workflow_alias}/{workflow_version}/execute",
                     json=workflow_input,
-                    headers=self.build_headers(),
+                    headers=headers,
                     timeout=900
                 )
                 raise_for_status_improved(response)
